package com.ticketmaster.presence.demo;

import android.content.DialogInterface;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.lifecycle.MutableLiveData;

import com.ticketmaster.presencesdk.PresenceModuleDelegate;
import com.ticketmaster.presencesdk.PresenceSDK;
import com.ticketmaster.presencesdk.event_tickets.DirectionsModule;
import com.ticketmaster.presencesdk.event_tickets.ModuleBase;
import com.ticketmaster.presencesdk.event_tickets.SeatUpgradesModule;
import com.ticketmaster.presencesdk.login.PresenceSdkConfigListener;
import com.ticketmaster.presencesdk.login.PresenceSimpleLoginListener;
import com.ticketmaster.presencesdk.login.TMLoginApi;
import com.ticketmaster.presencesdk.venuenext.VenueNextModule;

import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;

import kotlin.Unit;

public class PresenceSdkHostActivity extends AppCompatActivity implements PresenceSdkConfigListener {

    private static final String TAG = PresenceSdkHostActivity.class.getSimpleName();

    private final PresenceSimpleLoginListener mSimpleLoginListener = new PresenceSimpleLoginListener() {

        @Override
        public void onLoginSuccessful(TMLoginApi.BackendName backendName, String accessToken) {
            super.onLoginSuccessful(backendName, accessToken);
            Log.d(TAG, "onLoginSuccessful() called with: backendName = ["
                    + backendName + "], accessToken = [" + accessToken + "]");
        }

        @Override
        public void onLoginCancelled(TMLoginApi.BackendName backendName) {
            super.onLoginCancelled(backendName);
            Log.d(TAG, "onLoginCancelled() called with: backendName = [" + backendName + "]");
        }
    };


    private ConstraintLayout mGettingStartedContainer;
    private AlertDialog mProgressDialog;
    private boolean mConfigSuccessFul;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_psdk_host);
        mGettingStartedContainer = findViewById(R.id.getting_started_container);
        mGettingStartedContainer.setVisibility(View.VISIBLE);

        showLoading();

        // configure the Presence SDK
        configurePresenceSDK();
    }

    private void showLoading() {
        if (mProgressDialog == null) {
            mProgressDialog = new AlertDialog.Builder(this)
                    .setView(LayoutInflater.from(this).inflate(R.layout.layout_loading_view, null))
                    .setCancelable(false)
                    .create();
            mProgressDialog.setCanceledOnTouchOutside(false);
        }
        mProgressDialog.show();
    }

    private void configurePresenceSDK() {

        // Attach this Activity to receive callbacks for the configuration listener
        PresenceSDK.getPresenceSDK(getApplicationContext()).registerConfigListener(this);

        // Configure your branding color for the SDK
        final int brandingColor = Color.parseColor(BuildConfig.BRANDING_COLOR);
        PresenceSDK.getPresenceSDK(getApplicationContext())
                .setBrandingColors(brandingColor, brandingColor , brandingColor);

        // Set the configuration
        PresenceSDK.getPresenceSDK(getApplicationContext()).setConfig(
                BuildConfig.CONSUMER_KEY, /* Consumer key provided on dev portal */
                BuildConfig.TEAM_NAME, /* your team display name */
                BuildConfig.USE_NAM,
                true,
                false,
                PresenceSDK.HostEnvironment.US
        ); /* true for new account manager or false for classic */

        //setVenueNext();
    }

    @Override
    public void onPresenceSdkConfigSuccessful() {
        Log.d(TAG, "onPresenceSdkConfigSuccessful() called");
        mConfigSuccessFul = true;
        invalidateOptionsMenu();
        if (mProgressDialog != null) {
            mProgressDialog.dismiss();
        }
        mGettingStartedContainer.setVisibility(View.GONE);

        // unregister the configuration listener
        PresenceSDK.getPresenceSDK(getApplicationContext()).unregisterConfigListener(PresenceSdkHostActivity.this);

        // start the SDK when configuration is successful
        PresenceSDK.getPresenceSDK(getApplicationContext())
                .start(PresenceSdkHostActivity.this, R.id.presenceSDKView, mSimpleLoginListener);
    }

    @Override
    public void onPresenceSdkConfigFailed(String errorMessage) {
        Log.d(TAG, "onPresenceSdkConfigFailed() called with: errorMessage = [" + errorMessage + "]");
        mConfigSuccessFul = false;
        invalidateOptionsMenu();
        if (mProgressDialog != null) {
            mProgressDialog.dismiss();
        }

        // unregister the configuration listener
        PresenceSDK.getPresenceSDK(getApplicationContext()).unregisterConfigListener(PresenceSdkHostActivity.this);

        final AlertDialog.Builder builder = new AlertDialog.Builder(PresenceSdkHostActivity.this)
                .setTitle(R.string.configuration_error_title)
                .setMessage(R.string.configuration_error_message)
                .setPositiveButton(R.string.retry, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        showLoading();
                        configurePresenceSDK();
                    }
                })
                .setNegativeButton(R.string.cancel, null)
                .setCancelable(false);

        final AlertDialog alertDialog = builder.create();
        alertDialog.setCanceledOnTouchOutside(false);
        alertDialog.show();
    }

    /**
     * This is only an example of how to integrate Venue Next SDK with the Presence SDK.
     * You don't need to do it, if it's not necessary for your project.
     * You should have enabled custom modules based on your consumer key, if not ask for help in
     * TM support.
     */
    VenueNextModule venueNextModule = new VenueNextModule.Builder("venueMap.getInstance()").build();
    private void setVenueNext() {
        // Example
        /*List<String> ids = Collections.singletonList("pass_your_venue_id");
        VenueNextWeb.INSTANCE.retrieveExternalVenueMapping(this, ids, (venueMapping) -> {
            if (venueMapping.getVenues() != null && !venueMapping.getVenues().isEmpty()) {
                VenueMap venueMap = venueMapping.getVenues().get(0);
                if (venueMap.getInstance() != null) {
                    venueNextModule = new VenueNextModule.Builder(venueMap.getInstance()).build();
                }
            }
            return Unit.INSTANCE;
        });*/

        PresenceSDK.getPresenceSDK(getApplicationContext()).setPresenceModuleDelegate(presenceOrder -> {
            MutableLiveData<List<ModuleBase>> liveData = new MutableLiveData<>();
            ArrayList<ModuleBase> modules = new ArrayList<>();
            if (venueNextModule != null) {
                modules.add(getModuleBaseVenueNextView());
            }
            modules.add(getDirectionsModule(presenceOrder.getOrderInfo().getLatLng()));
            modules.add(new SeatUpgradesModule(this).build());
            liveData.setValue(modules);
            return liveData;
        });
    }

    private ModuleBase getModuleBaseVenueNextView() {
        return venueNextModule.createVenueNextView(this, venueNextViewClick -> {
            /*
            Here you can receive VenueNextViewClick to choose what to do based on
            venueNextViewClick.getVenueNextAction() & venueNextViewClick.getInstance(),
            like the example below:
            VenueNextActivity.Companion.newInstance(
                   this,
                    venueNextViewClick.getInstance(),
                    venueNextViewClick.getVenueNextAction()
            );
             */
            return Unit.INSTANCE;
        });
    }

    private ModuleBase getDirectionsModule(
            @Nullable PresenceModuleDelegate.LatLng latLng
    ) {
        return new DirectionsModule(
                this,
                latLng == null ? null : latLng.getLatitude(),
                latLng == null ? null : latLng.getLongitude()
        ).build();
    }


    @Override
    protected void onPause() {
        super.onPause();
        PresenceSDK.getPresenceSDK(getApplicationContext()).unregisterConfigListener(this);
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (!PresenceSDK.getPresenceSDK(this).isLoggedIn() && !mConfigSuccessFul) {
            mGettingStartedContainer.setVisibility(View.VISIBLE);
        } else {
            mGettingStartedContainer.setVisibility(View.GONE);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {

        MenuItem logoutMenu = menu.findItem(R.id.action_logout);
        MenuItem loginMenu = menu.findItem(R.id.action_login);
        if (PresenceSDK.getPresenceSDK(getApplicationContext()).isLoggedIn()) {
            logoutMenu.setVisible(true);
            loginMenu.setVisible(false);
        } else if (!mConfigSuccessFul) {
            loginMenu.setVisible(false);
            logoutMenu.setVisible(false);
        } else {
            logoutMenu.setVisible(false);
            loginMenu.setVisible(true);
        }

        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        int id = item.getItemId();
        if (id == R.id.action_logout) {
            invalidateOptionsMenu();
            PresenceSDK.getPresenceSDK(getApplicationContext()).logOut();
            configurePresenceSDK();

            return true;
        } else if (id == R.id.action_login) {

            mGettingStartedContainer.setVisibility(View.GONE);

            PresenceSDK.getPresenceSDK(getApplicationContext())
                    .start(PresenceSdkHostActivity.this, R.id.presenceSDKView, mSimpleLoginListener);

            return true;
        } else {
            return false;
        }
    }

}
